@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,6 +33,8 @@
 
 G_BEGIN_DECLS
 
+typedef struct _NMVpnPluginInfo NMVpnPluginInfo;
+
 typedef struct _NMVpnEditorPlugin NMVpnEditorPlugin;
 typedef struct _NMVpnEditor NMVpnEditor;
 
@@ -45,10 +46,9 @@ typedef NMVpnEditorPlugin * (*NMVpnEditorPluginFactory) (GError **error);
 NMVpnEditorPlugin *nm_vpn_editor_plugin_factory (GError **error);
 #endif
 
-
-/**************************************************/
+/*****************************************************************************/
 /* Editor plugin interface                        */
-/**************************************************/
+/*****************************************************************************/
 
 #define NM_TYPE_VPN_EDITOR_PLUGIN               (nm_vpn_editor_plugin_get_type ())
 #define NM_VPN_EDITOR_PLUGIN(obj)               (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_VPN_EDITOR_PLUGIN, NMVpnEditorPlugin))
@@ -80,6 +80,8 @@ typedef enum /*< flags >*/ {
 /* D-Bus service name of the plugin's VPN service */
 #define NM_VPN_EDITOR_PLUGIN_SERVICE "service"
 
+typedef struct _NMVpnEditorPluginVT NMVpnEditorPluginVT;
+
 /**
  * NMVpnEditorPluginInterface:
  * @g_iface: the parent interface
@@ -97,6 +99,9 @@ typedef enum /*< flags >*/ {
  * @get_suggested_filename: For a given connection, return a suggested file
  *   name.  Returned value will be %NULL or a suggested file name to be freed by
  *   the caller.
+ * @notify_plugin_info_set: A callback to be called when the plugin info is set.
+ * @get_vt: return a virtual function table to implement further functions in
+ *   the plugin, without requiring to update libnm. Used by nm_vpn_editor_plugin_get_vt().
  *
  * Interface for VPN editor plugins.
  */
@@ -119,6 +124,12 @@ typedef struct {
 	                            GError **error);
 
 	char * (*get_suggested_filename) (NMVpnEditorPlugin *plugin, NMConnection *connection);
+
+	void (*notify_plugin_info_set) (NMVpnEditorPlugin *plugin,
+	                                NMVpnPluginInfo *plugin_info);
+
+	const NMVpnEditorPluginVT *(*get_vt) (NMVpnEditorPlugin *plugin,
+	                                      gsize *out_vt_size);
 } NMVpnEditorPluginInterface;
 
 GType nm_vpn_editor_plugin_get_type (void);
@@ -129,6 +140,11 @@ NMVpnEditor *nm_vpn_editor_plugin_get_editor (NMVpnEditorPlugin *plugin,
 
 NMVpnEditorPluginCapability nm_vpn_editor_plugin_get_capabilities (NMVpnEditorPlugin *plugin);
 
+NM_AVAILABLE_IN_1_4
+gsize nm_vpn_editor_plugin_get_vt (NMVpnEditorPlugin *plugin,
+                                   NMVpnEditorPluginVT *vt,
+                                   gsize vt_size);
+
 NMConnection *nm_vpn_editor_plugin_import                 (NMVpnEditorPlugin *plugin,
                                                            const char *path,
                                                            GError **error);
@@ -147,6 +163,18 @@ NMVpnEditorPlugin *nm_vpn_editor_plugin_load_from_file  (const char *plugin_name
                                                          gpointer user_data,
                                                          GError **error);
 
+NM_AVAILABLE_IN_1_4
+NMVpnEditorPlugin *nm_vpn_editor_plugin_load (const char *plugin_name,
+                                              const char *check_service,
+                                              GError **error);
+
+NM_AVAILABLE_IN_1_4
+NMVpnPluginInfo *nm_vpn_editor_plugin_get_plugin_info (NMVpnEditorPlugin *plugin);
+NM_AVAILABLE_IN_1_4
+void             nm_vpn_editor_plugin_set_plugin_info (NMVpnEditorPlugin *plugin, NMVpnPluginInfo *plugin_info);
+
+#include "nm-vpn-plugin-info.h"
+
 G_END_DECLS
 
-#endif	/* __NM_VPN_EDITOR_PLUGIN_H__ */
+#endif /* __NM_VPN_EDITOR_PLUGIN_H__ */
