@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,14 +19,11 @@
 
 #include "nm-default.h"
 
-#include <string.h>
+#include "nm-device-vxlan.h"
 
 #include "nm-setting-connection.h"
 #include "nm-setting-vxlan.h"
 #include "nm-utils.h"
-
-#include "nm-device-vxlan.h"
-#include "nm-device-private.h"
 #include "nm-object-private.h"
 
 G_DEFINE_TYPE (NMDeviceVxlan, nm_device_vxlan, NM_TYPE_DEVICE)
@@ -95,7 +91,7 @@ nm_device_vxlan_get_hw_address (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return NM_DEVICE_VXLAN_GET_PRIVATE (device)->hw_address;
+	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->hw_address);
 }
 
 /**
@@ -162,7 +158,7 @@ nm_device_vxlan_get_group (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return NM_DEVICE_VXLAN_GET_PRIVATE (device)->group;
+	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->group);
 }
 
 /**
@@ -178,7 +174,7 @@ nm_device_vxlan_get_local (NMDeviceVxlan *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_VXLAN (device), NULL);
 
-	return NM_DEVICE_VXLAN_GET_PRIVATE (device)->local;
+	return nm_str_not_empty (NM_DEVICE_VXLAN_GET_PRIVATE (device)->local);
 }
 
 /**
@@ -362,7 +358,7 @@ nm_device_vxlan_get_l2miss (NMDeviceVxlan *device)
  * nm_device_vxlan_get_l3miss:
  * @device: a #NMDeviceVxlan
  *
- * Returns: wether netlink IP ADDR miss notifications are generated
+ * Returns: whether netlink IP ADDR miss notifications are generated
  *
  * Since: 1.2
  **/
@@ -410,12 +406,11 @@ get_hw_address (NMDevice *device)
 	return nm_device_vxlan_get_hw_address (NM_DEVICE_VXLAN (device));
 }
 
-/***********************************************************/
+/*****************************************************************************/
 
 static void
 nm_device_vxlan_init (NMDeviceVxlan *device)
 {
-	_nm_device_set_device_type (NM_DEVICE (device), NM_DEVICE_TYPE_VXLAN);
 }
 
 static void
@@ -542,8 +537,6 @@ nm_device_vxlan_class_init (NMDeviceVxlanClass *vxlan_class)
 
 	g_type_class_add_private (vxlan_class, sizeof (NMDeviceVxlanPrivate));
 
-	_nm_object_class_add_interface (nm_object_class, NM_DBUS_INTERFACE_DEVICE_VXLAN);
-
 	/* virtual methods */
 	object_class->finalize = finalize;
 	object_class->get_property = get_property;
