@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* NetworkManager -- Network link manager
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,10 +21,8 @@
 
 #include "nm-firewall-manager.h"
 
-#include <string.h>
-
 #include "NetworkManagerUtils.h"
-#include "nm-utils/c-list.h"
+#include "c-list/src/c-list.h"
 
 /*****************************************************************************/
 
@@ -174,7 +171,7 @@ _cb_info_create (NMFirewallManager *self,
 
 	if (priv->running || priv->proxy_cancellable) {
 		info->mode_mutable = CB_INFO_MODE_DBUS_WAITING;
-		info->dbus.arg = g_variant_new ("(ss)", zone ? zone : "", iface);
+		info->dbus.arg = g_variant_new ("(ss)", zone ?: "", iface);
 	} else
 		info->mode_mutable = CB_INFO_MODE_IDLE;
 
